import 'package:osam/osam.dart';
import 'package:osam_flutter/src/navigation/osam_route.dart';

import '../../osam_flutter.dart';

part 'navigation.g.dart';

class NoRouteException implements Exception {}

@HiveType(typeId: 222)
class Navigation {
  @HiveField(0)
  final List<OsamRoute<Object, Object>> _routes;
  PropertyNotifier<List<OsamRoute<Object, Object>>> _notifier;
  IProperty<List<OsamRoute<Object, Object>>> get routes => _notifier;

  Navigation(this._routes) {
    _notifier = PropertyNotifier(_routes);
  }

  Future<R> push<A, R>(OsamRoute<A, R> route) {
    _notifier.update((value) => value.add(route));
    return route.whenComplete;
  }

  void pop() => _notifier.update((value) => value.removeLast());

  T findRoute<T extends OsamRoute>() =>
      _routes.firstWhere((element) => element is T);

  Future<R> pushReplacement<A, R>(OsamRoute<A, R> route) {
    _notifier.update((value) {
      value.removeLast();
      value.add(route);
    });
    return route.whenComplete;
  }

  Future<R> pushAndRemoveUntil<A, R>(OsamRoute<A, R> route) {
    _notifier.update((value) {
      value.clear();
      value.add(route);
    });
    return route.whenComplete;
  }

  void popUntil<T extends OsamRoute>() => _notifier.update((value) {
        if (value.contains(T)) {
          while (!(value.last is T)) {
            value.removeLast();
          }
        }
      });
}

@HiveType(typeId: 221)
class TabNavigation {
  @HiveField(0)
  String _tab;
  PropertyNotifier<String> _notifier;
  IProperty<String> get tabNotifier => _notifier;

  TabNavigation(this._tab) {
    _notifier = PropertyNotifier(_tab);
  }

  set tab(String tab) {
    _tab = tab;
    _notifier.value = tab;
  }
}
